﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectBulldozer.Drawnings;
using ProjectBulldozer.Exceptions;

namespace ProjectBulldozer.CollectionGenericObjects;

/// <summary>
/// Класс-хранилище коллекций
/// </summary>
/// <typeparam name="T"></typeparam>
public class StorageCollection<T> 
    where T : DrawningDozer
{
    /// <summary>
    /// Словарь (хранилище) с коллекциями
    /// </summary>
    readonly Dictionary<string, ICollectoinGenericObjects<T>> _storages;

    /// <summary>
    /// Возвращение списка названий коллекций
    /// </summary>
    public List<string> Keys => _storages.Keys.ToList();

    /// <summary>
    /// Ключевое слово, с которого должен начианться файл
    /// </summary>
    private readonly string _collectionKey = "CollectionStorage";

    /// <summary>
    /// Разделитель для записи ключа и значения элемента словаря
    /// </summary>
    private readonly string _separatorForKeyValue = "|";

    /// <summary>
    /// Разделительдля записи коллекций данных в файл
    /// </summary>
    private readonly string _separatorItems = ";";

    /// <summary>
    /// Конструктор
    /// </summary>
    public StorageCollection() 
    {
        _storages = new Dictionary<string, ICollectoinGenericObjects<T>>();
    }

    /// <summary>
    /// Добавление коллекции в хранилище
    /// </summary>
    /// <param name="name">название коллекции</param>
    /// <param name="collectionType">тип коллекции</param>
    public void AddCollection(string name, CollectionType collectionType)
    {
        // Проверка, что name не пустой и нет в словаре записи с таким ключом.
        if (_storages.ContainsKey(name) && name == "")
        {
            return;
        }
        // Логика для добавления.
        if (collectionType == CollectionType.Massive)
        {
            _storages[name] = new MassiveGenericObject<T>();
        }
        else
        {
            _storages[name] = new ListGenericObjects<T>();
        }
    }

    /// <summary>
    /// Удаление коллекции
    /// </summary>
    /// <param name="name">Название коллекции</param>
    public void DelCollection(string name)
    {
        // Логика для удаления коллекции.
        if (_storages.ContainsKey(name))
        {
            _storages.Remove(name);
        }
    }

    /// <summary>
    /// Доступ к коллекции
    /// </summary>
    /// <param name="name">Название коллекции</param>
    /// <returns></returns>
    public ICollectoinGenericObjects<T>? this[string name]
    {
        get
        {
            // Логика получения объекта.
            if (_storages.ContainsKey(name))
            {
                return _storages[name];
            }
            return null;
        }
    }

    /// <summary>
    /// Сохранение информации по бульдозерам в хранилище в файла
    /// </summary>
    /// <param name="filename">Путь и имя файла</param>
    public void SaveData(string filename)
    {
        if (_storages.Count == 0)
        {
            throw new Exception("В хранилище отсутсвует коллекция для сохранения!");
        }

        if (File.Exists(filename)) 
        {
            File.Delete(filename); 
        }

        StringBuilder stringBuilder = new();

        stringBuilder.Append(_collectionKey);
        foreach (KeyValuePair<string, ICollectoinGenericObjects<T>> value in _storages)
        {
            stringBuilder.Append(Environment.NewLine);
            // не сохраняем пустые коллекции.
            if (value.Value.Count == 0)
            {
                continue;
            }

            stringBuilder.Append(value.Key);
            stringBuilder.Append(_separatorForKeyValue);
            stringBuilder.Append(value.Value.GetCollectionType);
            stringBuilder.Append(_separatorForKeyValue);
            stringBuilder.Append(value.Value.MaxCount);
            stringBuilder.Append(_separatorForKeyValue);

            foreach (T? item in value.Value.GetItems())
            {
                string data = item?.GetDataForSave() ?? string.Empty;
                if (string.IsNullOrEmpty(data))
                {
                    continue;
                }

                stringBuilder.Append(data);
                stringBuilder.Append(_separatorItems);
            }
        }

        using FileStream fileStream = new(filename, FileMode.Create);
        byte[] info = new UTF8Encoding(true).GetBytes(stringBuilder.ToString());
        fileStream.Write(info, 0, info.Length);
    }

    /// <summary>
    /// Загрузка информации по бульдозерам в хранилище из файла
    /// </summary>
    /// <param name="filename">Путь и имя файла</param>
    public void LoadData(string filename)
    {
        if (!File.Exists(filename))
        {
            throw new Exception("Файл не существует!");
        }

        string bufferTextfromFile = "";
        using (FileStream fileStream = new(filename, FileMode.Open))
        {
            byte[] bytes = new byte[fileStream.Length];
            UTF8Encoding temp = new(true);
            while (fileStream.Read(bytes, 0, bytes.Length) > 0) 
            {
                bufferTextfromFile += temp.GetString(bytes);
            }
        }

        string[] strs = bufferTextfromFile.Split(new char[] { '\r', '\n'},
            StringSplitOptions.RemoveEmptyEntries);
        if (strs == null || strs.Length == 0)
        {
            throw new Exception("В файле нет данных!");
        }

        if (!strs[0].Equals(_collectionKey))
        {
            throw new Exception("В файле неверные данные!");
        }

        _storages.Clear();
        foreach (string data in strs)
        {
            string[] record = data.Split(_separatorForKeyValue, 
                StringSplitOptions.RemoveEmptyEntries);
            if (record.Length != 4)
            {
                continue;
            }

            CollectionType collectionType = (CollectionType)Enum.Parse(typeof(CollectionType), record[1]);
            ICollectoinGenericObjects<T>? collectoin = StorageCollection<T>.CreateCollection(collectionType) ?? 
                throw new Exception("Не удалось определить тип коллекции: " + record[1]);

            collectoin.MaxCount = Convert.ToInt32(record[2]);

            string[] set = record[3].Split(_separatorItems, StringSplitOptions.RemoveEmptyEntries);

            foreach (string elem in set)
            {
                if (elem?.CreateDrawningDozer() is T dozer)
                {
                    try
                    {
                        if (collectoin.Insert(dozer) == -1)
                        {
                            throw new Exception("Объект не удалось добавить в коллекцию: " + record[3]);
                        }
                    }
                    catch (CollectionOverflowException exception)
                    {
                        throw new Exception("Коллекция переполнена!", exception);
                    }
                }
            }
            _storages.Add(record[0], collectoin);
        }
    }


    private static ICollectoinGenericObjects<T>? CreateCollection(CollectionType collectionType)
    {
        return collectionType switch
        {
            CollectionType.Massive => new MassiveGenericObject<T>(),
            CollectionType.List => new ListGenericObjects<T>(),
            _ => null,
        };
    }
}
