﻿using Microsoft.Extensions.Logging;
using ProjectBulldozer.CollectionGenericObjects;
using ProjectBulldozer.Drawnings;
using ProjectBulldozer.Exceptions;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectBulldozer;

/// <summary>
/// Форма работы с компанией и её коллекцией
/// </summary>
public partial class FormBulldozerCollection : Form
{
    /// <summary>
    /// Хранилище коллекций
    /// </summary>
    private readonly StorageCollection<DrawningDozer> _storageCollection;

    /// <summary>
    /// Компания
    /// </summary>
    private AbstractCompany? _company = null;

    private readonly ILogger _logger;

    /// <summary>
    /// Конструктор
    /// </summary>
    public FormBulldozerCollection(ILogger<FormBulldozerCollection> logger)
    {
        InitializeComponent();
        _storageCollection = new();
        _logger = logger;
    }

    /// <summary>
    /// Выбор компании
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ComboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
    {
        panelCompanyTools.Enabled = false;

    }

    /// <summary>
    /// Добавление бульдозера
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAddDozer_Click(object sender, EventArgs e)
    {
        FormBulldozerConfig form = new();
        // Передача метода.
        form.Show();
        form.AddEvent(SetDozer);
    }

    /// <summary>
    /// Добавление бульдозера в коллекцию
    /// </summary>
    /// <param name="dozer"></param>
    private void SetDozer(DrawningDozer? dozer)
    {
        try
        {
            if (_company == null || dozer == null)
            {
                return;
            }

            if (_company + dozer != -1)
            {
                MessageBox.Show("Объект добавлен.");
                pictureBox.Image = _company.Show();
                _logger.LogInformation("Объект добавлен: " + dozer.GetDataForSave());
            }
        }
        catch (CollectionOverflowException exception)
        {
            MessageBox.Show(exception.Message);
            _logger.LogError("Ошибка: превышено допустимое коллическтво! {Message}", exception);

        }
    }

    /// <summary>
    /// Создание объекта класса-перемещения
    /// </summary>
    /// <param name="type">Тип создаваемого объекта</param>
    private void CreateObject(string type)
    {
        if (_company == null)
        {
            return;
        }

        Random random = new();
        DrawningDozer drawningDozer;
        switch (type)
        {
            case nameof(DrawningDozer):
                drawningDozer = new DrawningDozer(random.Next(100, 300), random.Next(1000, 3000),
            GetBodyColor(random));
                break;
            case nameof(DrawningBulldozer):
                //Вызов диалогового окна для выбора цвета
                drawningDozer = new DrawningBulldozer(random.Next(100, 300), random.Next(1000, 3000),
                    GetBodyColor(random), GetWheelsColor(random),
                    Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                break;
            default:
                return;
        }

        if (_company + drawningDozer != -1)
        {
            MessageBox.Show("Объект добавлен.");
            pictureBox.Image = _company.Show();
        }
        else
        {
            MessageBox.Show("Не удалось добавить объект...");
        }
    }

    /// <summary>
    /// Получение цвета корпуса бульдозера
    /// </summary>
    /// <param name="random">Генератор случайных чисел</param>
    /// <returns></returns>
    private static Color GetBodyColor(Random random)
    {
        Color color = Color.FromArgb(random.Next(170, 256), random.Next(170, 256), random.Next(30, 140));
        ColorDialog dialog = new();
        if (dialog.ShowDialog() == DialogResult.OK)
        {
            color = dialog.Color;
        }
        return color;
    }

    /// <summary>
    /// Получение цвета гусеницы бульдозера
    /// </summary>
    /// <param name="random">Генератор случайных чисел</param>
    /// <returns></returns>
    private static Color GetWheelsColor(Random random)
    {
        Color color = Color.FromArgb(random.Next(30, 120), random.Next(30, 120), random.Next(30, 120));
        ColorDialog dialog = new();
        if (dialog.ShowDialog() == DialogResult.OK)
        {
            color = dialog.Color;
        }
        return color;
    }

    /// <summary>
    /// Удаление объекта
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonDelBulldozer_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(maskedTextBox.Text) || _company == null)
        {
            _logger.LogWarning("Удаление объекта из несуществующей коллекции!");
            return;
        }

        if (MessageBox.Show("Удалить объект?", "Удаление...", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
        {
            return;
        }

        int pos = Convert.ToInt32(maskedTextBox.Text);

        try
        {
            if (_company - pos != null)
            {
                MessageBox.Show("Объект удалён.");
                pictureBox.Image = _company.Show();
                _logger.LogInformation("Объект удалён: " + pos);
            }
        }
        catch (ObjectNotFoundException exception)
        {
            MessageBox.Show(exception.Message);
            _logger.LogWarning("Попытка удалить не найденный объект по позиции: " + pos);
        }
        catch (PositionOutOfCollectionException)
        {
            MessageBox.Show("Объект вне коллекции!");
            _logger.LogWarning("Попытка удалить объект вне коллекции: " + pos);
        }
    }

    /// <summary>
    /// Передача объека в другую форму
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonGoToCheck_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        DrawningDozer? dozer = null;
        int counter = 100;

        try
        {
            while (dozer == null)
            {
                dozer = _company.GetRandomObject();
                counter--;
                if (counter <= 0)
                {
                    break;
                }
            }

            if (dozer == null)
            {
                return;
            }

            FormBulldozer form = new()
            {
                SetCar = dozer
            };
            form.ShowDialog();
        }
        catch (Exception exception)
        {
            MessageBox.Show(exception.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    /// <summary>
    /// Прорисовка коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonRefresh_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        try
        {
            pictureBox.Image = _company.Show();
        }
        catch (PositionOutOfCollectionException) { }
        catch (Exception) { }
    }

    /// <summary>
    /// Добавление коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCollectionAdd_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonList.Checked && !radioButtonMassive.Checked))
        {
            MessageBox.Show("Не все данные заполнены!", "Ошибка!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            return;
        }

        try
        {
            CollectionType collectionType = CollectionType.None;
            if (radioButtonMassive.Checked)
            {
                collectionType = CollectionType.Massive;
            }
            else if (radioButtonList.Checked)
            {
                collectionType = CollectionType.List;
            }
            _storageCollection.AddCollection(textBoxCollectionName.Text, collectionType);
            RefreshListBoxItems();
            _logger.LogInformation("Коллекция добавлена " + textBoxCollectionName.Text);
        }
        catch (Exception ex)
        {
            _logger.LogError("Ошибка: {Message}", ex.Message);
        }

    }


    /// <summary>
    /// Удаление коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCollectionDel_Click(object sender, EventArgs e)
    {
        // Логика удаления элемента из коллекции.
        // Нужно убедиться, что есть выбранная коллекция
        if (listBoxCollection.SelectedItem == null || listBoxCollection.SelectedIndex < 0)
        {
            MessageBox.Show("Коллекция не выбрана!");
            return;
        }
        try
        {
            // Спросить у пользователя через MessageBox, что он подтверждает, что хочет удалить запись
            if (MessageBox.Show("Удалить коллекцию?", "Удаление...", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            // Удалить и обновить ListBox
            //_storageCollection.DelCollection(listBoxCollection.SelectedItem.ToString());
            _logger.LogInformation("Коллекция удалена: " + listBoxCollection.SelectedItem.ToString());
            RefreshListBoxItems();
        }
        catch (Exception ex)
        {
            _logger.LogError("Ошибка: {Message}", ex.Message);
        }
        



    }

    /// <summary>
    /// Обновление списка в ListBoxCollection
    /// </summary>
    private void RefreshListBoxItems()
    {
        listBoxCollection.Items.Clear();
        for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
        {
            string? colName = _storageCollection.Keys?[i];
            if (!string.IsNullOrEmpty(colName))
            {
                listBoxCollection.Items.Add(colName);
            }
        }
    }

    /// <summary>
    /// Создание компании
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCreateCompany_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана!");
            return;
        }

        ICollectoinGenericObjects<DrawningDozer>? collection = _storageCollection[listBoxCollection.SelectedItem.ToString() ?? string.Empty];
        if (collection == null)
        {
            MessageBox.Show("Коллекция не проинициализирована!");
            return;
        }

        try
        {
            switch (comboBoxSelectorCompany.Text)
            {
                case "Хранилище":
                    _company = new Garage(pictureBox.Width, pictureBox.Height, collection);
                    break;
            }
        }
        catch (ObjectNotFoundException) { }

        panelCompanyTools.Enabled = true;
        RefreshListBoxItems();
    }

    /// <summary>
    /// Обработка нажатия "Сохранить"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (saveFileDialog.ShowDialog() == DialogResult.OK)
        {
            try
            {
                _storageCollection.SaveData(saveFileDialog.FileName);
                MessageBox.Show("Сохранение прошло успешно.", "Результат",
                    MessageBoxButtons.OK, MessageBoxIcon.Information);
                _logger.LogInformation("Сохранение в файл: {filename}", saveFileDialog.FileName);
            }
            catch (Exception exception)
            {
                MessageBox.Show("Не сохранено!", "Результат",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", exception.Message);
            }
        }
    }

    /// <summary>
    /// Обрработка нажати "Загрузка"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
    {
        // Логика
        if (openFileDialog.ShowDialog() == DialogResult.OK)
        {
            try
            {
                _storageCollection.LoadData(openFileDialog.FileName);
                MessageBox.Show("Загрузка прошла успешно.", "Результат",
                    MessageBoxButtons.OK, MessageBoxIcon.Information);
                foreach (var collection in _storageCollection.Keys)
                {
                    listBoxCollection.Items.Add(collection);
                }
                _logger.LogInformation("Загрузка из файла: {filename}", saveFileDialog.FileName);
                RefreshListBoxItems();
            }
            catch (Exception exception)
            {
                MessageBox.Show("Загрузка не удалась!", "Результат",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", exception.Message);
            }
        }
        RefreshListBoxItems();
    }
}
